# pyqt modules
from PyQt4.QtGui import (QApplication, QLabel, QListView, QMenu, 
                         QProgressDialog, QPushButton, QMainWindow, QVBoxLayout, 
                         QWidget)
from PyQt4.QtCore import (pyqtSignature, Qt, SIGNAL)

# third-party modules
import config as c

# program modules 
from ui.ui_exercises_window import Ui_ExercisesWindow
import routines_window as ww
import exercise_wizard as ew
import target_areas_window as taw

from delegates import ExercisesDelegate

class ExercisesWindow(QMainWindow, Ui_ExercisesWindow):

    def __init__(self, parent=None, database=None, target_area=None):
        super(ExercisesWindow, self).__init__(parent)
        self.setAttribute(Qt.WA_Maemo5StackedWindow)

        self.setupUi(self)
        self.config = c.Config()
        self.db = database
        self.target_area = target_area
        self.setupTableView()

        self.customContextMenuRequested.connect(self.openMenu)

    def setupTableView(self):
        if self.target_area:
            filtered_exercises_model = self.db.exercises_model
            filtered_exercises_model.setFilter("t_name='%s'" % self.target_area)
            filtered_exercises_model.select()

            self.exercises_list_view.setModel(filtered_exercises_model)
            self.target_areas_action.setEnabled(False)
            self.new_exercise_button.hide()
            self.setWindowTitle(self.target_area)
        else:
            self.exercises_list_view.setModel(self.db.exercises_model)

        self.exercises_list_view.setModelColumn(1)
        self.exercises_list_view.setItemDelegate(ExercisesDelegate())

    @pyqtSignature('bool')
    def on_routines_action_triggered(self, checked):
        self.config.interface['last-fitness'] = 'routines'
        self.parent().setFitnessWindow()
        routines_window = ww.RoutinesWindow(self.parent(), self.db)
        routines_window.show()
        self.hide()

    @pyqtSignature('bool')
    def on_target_areas_action_triggered(self, checked):
        self.config.interface['last-fitness'] = 'target_areas'
        self.parent().setFitnessWindow()
        target_areas_window = taw.TargetAreasWindow(self.parent(), self.db)
        target_areas_window.show()
        self.hide()

    def on_new_exercise_button_pressed(self):
        exercise_wizard = ew.ExerciseWizard(self, self.db)
        exercise_wizard.show()

    @pyqtSignature('QModelIndex')
    def on_exercises_list_view_clicked(self, index):
        pass

    def openMenu(self, position):
        menu = QMenu(self)
        edit_action = menu.addAction('Edit')
        delete_action = menu.addAction('Delete')

        action = menu.exec_(self.mapToGlobal(position))

        if action == edit_action:
            index = self.exercises_list_view.currentIndex()
            name = self.db.exercises_model.record(
                                       index.row()).value('ex_name')

            exercise_wizard = ew.ExerciseWizard(self, self.db, name)
            exercise_wizard.setStartId(1)
            exercise_wizard.setWindowTitle('Edit Exercise')

            description = self.db.exercises_model.record(
                                index.row()).value('ex_description')

            t_name = self.db.exercises_model.record(
                                          index.row()).value('t_name')
            t_id = self.db.getRow('t_name', t_name)

            f_name = self.db.exercises_model.record(
                                          index.row()).value('f_name')
            f_id = self.db.getRow('f_name', f_name)

            d_name = self.db.exercises_model.record(
                                          index.row()).value('d_name')
            d_id = self.db.getRow('d_name', d_name)

            i_name = self.db.exercises_model.record(
                                          index.row()).value('i_name')
            i_id = self.db.getRow('i_name', i_name)

            e_name = self.db.exercises_model.record(
                                          index.row()).value('e_name')
            e_id = self.db.getRow('e_name', e_name)
            

            """ REFERNCE
            target_area = self.db.exercises_model.index(index.row(),
                                                        3).data()
            frequency = self.db.exercises_model.index(index.row(),
                                                      4).data()
            duration = self.db.exercises_model.index(index.row(),
                                                    5).data()
            intensity = self.db.exercises_model.index(index.row(),
                                                      6).data()
            equipment = self.db.exercises_model.index(index.row(),
                                                      7).data()
            """

            media = self.db.exercises_model.index(index.row(),
                                                  8).data()


            exercise_wizard.name_line_edit.setText(name.toString())
            exercise_wizard.description_text_edit.setHtml(description.toString())
            exercise_wizard.target_area_selector.setCurrentIndex(t_id)
            exercise_wizard.frequency_selector.setCurrentIndex(f_id)
            exercise_wizard.duration_selector.setCurrentIndex(d_id)
            exercise_wizard.intensity_selector.setCurrentIndex(i_id)
            exercise_wizard.equipment_selector.setCurrentIndex(e_id)
            exercise_wizard.media_line_edit.setText(media.toString())

            exercise_wizard.show()
        elif action == delete_action:
            self.db.exercises_model.removeRow(
                                 self.exercises_list_view.currentIndex().row())
